@@ -94,7 +94,7 @@
 #define NONEXEC_STACK
 #endif
 
-#if defined(__x86_64__) && (defined(_WIN64) || defined (__CYGWIN__))
+#if (defined(__x86_64__) || defined(_M_X64)) && (defined(_WIN64) || defined (__CYGWIN__))
 #define IS_MSABI 1 /* Not using SYSV */
 #endif
 
